@@ -1,4 +1,7 @@
+""" Args for STFT """
+
 class LibrosaSTFTArgs(dict):
+    """ Class representing the kwargs for stft of librosa"""
     def __init__(self, **kwargs):
         # Default arguments for librosa.stft
         default_args = {
@@ -8,12 +11,12 @@ class LibrosaSTFTArgs(dict):
             'window': 'hann',
             'center': True,
         }
-        
+
         # Initialize with default arguments, then update with any user-provided arguments
         super().__init__({**default_args, **kwargs})
-        
+
         # Handle derived default values
         if self['hop_length'] is None:
             self['hop_length'] = self['n_fft'] // 4
         if self['win_length'] is None:
-            self['win_length'] = self['n_fft']
\ No newline at end of file
+            self['win_length'] = self['n_fft']
